/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.business;

import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;

import org.springmodules.jcr.JcrCallback;
import org.springmodules.jcr.JcrTemplate;

import sun.net.www.MimeTable;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;


/**
 * Implementation of IRepositoryFileDAO for Jackrabbit JCR
 */
public class JcrRepositoryFileDAO extends AbstractRepositoryDAO implements IRepositoryFileDAO
{
    public static final String PATH_SEPARATOR = "/";
    public static final String RESOURCE_TYPE = "JSR170_FILE";
    public static final String NODE_TYPE_FOLDER = "nt:folder";
    public static final String NODE_TYPE_FILE = "nt:file";
    public static final String NODE_TYPE_FILE_RESOURCE = "nt:resource";
    public static final String LUTECE_NAMESPACE = "lutece";
    public static final String LUTECE_NAMESPACE_URI = "http://lutece.paris.fr/lut/1.0";
    public static final String DEFAULT_WORKSPACE_NAME = "default";
    public static final String ATTRIBUTE_NODE_SIZE = "lutece:size";
    public static final String NODE_TYPE_JCR_CONTENT = "jcr:content";
    private static final String REGEXP_ABSOLUTE_PATH = "^/+";
    protected static final String MIXIN_REFERENCEABLE = "mix:referenceable";

    /** This class implements the Singleton design pattern. */
    private static JcrRepositoryFileDAO _dao;

    ///////////////////////////////////////////////////////////////////////////////////////
    // Constructor and instanciation methods

    /**
     * Returns the unique instance of the singleton.
     * @param jcrTemplate a jcrTemplate
     * @param repositoryInitializer a repositoryInitializer
     * @param strDefaultWorkspaceName a default workspace name
     * @return the instance
     */
    static IRepositoryFileDAO getInstance( JcrTemplate jcrTemplate, IRepositoryInitializer repositoryInitializer,
        String strDefaultWorkspaceName )
    {
        if ( _dao == null )
        {
            _dao = new JcrRepositoryFileDAO(  );
            _dao.init( jcrTemplate, repositoryInitializer, strDefaultWorkspaceName );
        }

        return _dao;
    }

    /**
     * Get the absolute path of a node
     * @param node the node
     * @return a string containing the absolute path or null if node doesn't exist
     */
    private String getAbsolutePath( Node node )
    {
        if ( node != null )
        {
            try
            {
                return node.getPath(  );
            }
            catch ( RepositoryException e )
            {
                e.printStackTrace(  );
            }
        }

        return null;
    }

    /**
     * Get the node content
     * @param node the node
     * @return an inputstream or null if content doesn't exist
     */
    private InputStream getFileContent( Node node )
    {
        return (InputStream) getFileProperty( node, "jcr:data", InputStream.class );
    }

    /**
     * Get the node name
     * @param node the node
     * @return the node name
     */
    private String getName( Node node )
    {
        String strName = null;

        try
        {
            strName = node.getName(  );
        }
        catch ( RepositoryException e )
        {
            AppLogService.debug( e );
        }

        return strName;
    }

    /**
     * Get the size of node content
     * @param node the node
     * @return the size in bytes
     */
    private long getSize( Node node )
    {
        long size = 0;

        try
        {
            size = node.getProperty( ATTRIBUTE_NODE_SIZE ).getLong(  );
        }
        catch ( RepositoryException e )
        {
            // nothing to do
            AppLogService.error( e.getLocalizedMessage(  ), e );
        }

        return size;
    }

    /**
     * Get the node type
     * @param node the node
     * @return true if it is a directory, false otherwise
     */
    private boolean isDirectory( Node node )
    {
        return isTypeOf( node, NODE_TYPE_FOLDER );
    }

    /**
     * Get the node type
     * @param node the node
     * @return true if it is a file, false otherwise
     */
    private boolean isFile( Node node )
    {
        return isTypeOf( node, NODE_TYPE_FILE );
    }

    /**
     * Get the node existence
     * @param node the node
     * @return true if node exists, false otherwise
     */
    private boolean exists( Node node )
    {
        return node != null;
    }

    /**
     * Get a node property of type clazz
     * @param node the node
     * @param strPropertyName the property name
     * @param clazz the type of object to return
     * @return an instance of clazz containing the value of the property strPropertyName
     */
    private Object getFileProperty( Node node, String strPropertyName, Class clazz )
    {
        try
        {
            if ( node.hasProperty( NODE_TYPE_FOLDER ) )
            {
                throw new AppException( "Not a file exception" );
            }

            Node resNode = node.getNode( NODE_TYPE_JCR_CONTENT );
            Value value = resNode.getProperty( strPropertyName ).getValue(  );

            if ( clazz.equals( String.class ) )
            {
                return value.getString(  );
            }
            else if ( clazz.equals( InputStream.class ) )
            {
                return value.getStream(  );
            }
            else if ( clazz.equals( Calendar.class ) )
            {
                return value.getDate(  );
            }
            else
            {
                throw new AppException( "Unsupported Node type" );
            }
        }
        catch ( RepositoryException e )
        {
            throw new AppException( e.getMessage(  ), e );
        }
    }

    /**
     * Get the Mime type
     * @param node the node
     * @return the mime type
     */
    private String getMimeType( Node node )
    {
        return (String) getFileProperty( node, "jcr:mimeType", String.class );
    }

    /**
     * Get the creation date
     * @param node the node
     * @return the creation date
     */
    private Calendar getCreated( Node node )
    {
        Calendar creationDate = null;

        try
        {
            creationDate = node.getProperty( "jcr:created" ).getDate(  );
        }
        catch ( RepositoryException e )
        {
            AppLogService.debug( e );
        }

        return creationDate;
    }

    /**
     * Get the modification date
     * @param node the node
     * @return the modification date
     */
    private Calendar getLastModified( Node node )
    {
        return (Calendar) getFileProperty( node, "jcr:lastModified", Calendar.class );
    }

    /**
     * Test the node type
     * @param node the node
     * @param strType a type
     * @return return true is node type is strType, false otherwise
     */
    private boolean isTypeOf( Node node, String strType )
    {
        try
        {
            return ( node != null ) && node.isNodeType( strType );
        }
        catch ( RepositoryException e )
        {
            throw new AppException( e.getMessage(  ), e );
        }
    }

    /**
     * Get a node by its path
     * @param session the session
     * @param strPath the absolute path
     * @return a node
     */
    private Node getNode( Session session, String strPath )
    {
        String strRelPath = ( strPath == null ) ? "" : strPath.replaceFirst( REGEXP_ABSOLUTE_PATH, "" );

        try
        {
            if ( "".equals( strRelPath ) )
            {
                // we're looking for the root node
                return session.getRootNode(  );
            }
            else
            {
                return session.getRootNode(  ).getNode( strRelPath );
            }
        }
        catch ( PathNotFoundException e )
        {
            throw new AppException( e.getMessage(  ), e );
        }
        catch ( RepositoryException e )
        {
            throw new AppException( e.getMessage(  ), e );
        }
    }

    /**
     * Set the content of a node
     * @param node the node
     * @param inputStream a stream with the content
     * @param strMimeType the Mime type
     * @return true if the transfert is ok and node is a file, false otherwise
     */
    private boolean setContent( Node node, InputStream inputStream, String strMimeType )
    {
        try
        {
            if ( node.hasProperty( NODE_TYPE_FOLDER ) )
            {
                return false;
            }

            Node resNode = null;
            resNode = node.addNode( NODE_TYPE_JCR_CONTENT, NODE_TYPE_FILE_RESOURCE );
            resNode.setProperty( "jcr:mimeType", strMimeType );
            resNode.setProperty( "jcr:data", inputStream );
            resNode.setProperty( "jcr:lastModified", Calendar.getInstance(  ) );

            return true;
        }
        catch ( RepositoryException e )
        {
            throw new AppException( e.getMessage(  ), e );
        }
    }

    /**
     * @param strWorkspace the workspace name
     * @param strPath the absolute path to create
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#create(java.lang.String, java.lang.String)
     */
    public void create( String strWorkspace, final String strPath )
    {
        execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    String strRelPath = strPath.replaceFirst( REGEXP_ABSOLUTE_PATH, "" );
                    Node addedNode = session.getRootNode(  ).addNode( strRelPath, NODE_TYPE_FOLDER );
                    addedNode.addMixin( MIXIN_REFERENCEABLE );
                    session.save(  );

                    return null;
                }
            } );
    }

    /**
     * @param strWorkspace the workspace name
     * @param strFileName the absolute file name
     * @param file a File containing the file content
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#create(java.lang.String, java.lang.String, java.io.File)
     */
    public void create( String strWorkspace, final String strFileName, final File file )
    {
        execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    String strFilePath = strFileName.replaceAll( REGEXP_ABSOLUTE_PATH, "" );

                    Node addedNode = session.getRootNode(  ).addNode( strFilePath, "lutece:file" );
                    addedNode.setProperty( ATTRIBUTE_NODE_SIZE, file.length(  ) );
                    addedNode.addMixin( MIXIN_REFERENCEABLE );

                    MimeTable mt = MimeTable.getDefaultTable(  );
                    String mimeType = mt.getContentTypeFor( file.getName(  ) );

                    if ( mimeType == null )
                    {
                        mimeType = "application/octet-stream";
                    }

                    setContent( addedNode, new FileInputStream( file ), mimeType );
                    session.save(  );

                    return null;
                }
            } );
    }

    /**
     * @param strWorkspace the workspace name
     * @param strPath the absolute path of the element
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#delete(java.lang.String, java.lang.String)
     */
    public void delete( String strWorkspace, final String strPath )
    {
        execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    Node currentNode = getNode( session, strPath );
                    currentNode.remove(  );
                    session.save(  );

                    return null;
                }
            } );
    }

    /**
     * @param strWorkspace the workspace name
     * @param strPath the absolute directory path to search
     * @return a list of all IRepositoryFile in strPath
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#listFiles(java.lang.String, java.lang.String)
     */
    @SuppressWarnings( "unchecked" )
    public List<IRepositoryFile> listFiles( String strWorkspace, final String strPath )
    {
        return (List<IRepositoryFile>) execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    Node currentNode = getNode( session, strPath );

                    if ( currentNode == null )
                    {
                        return null;
                    }

                    ArrayList<IRepositoryFile> listFiles = new ArrayList<IRepositoryFile>(  );

                    NodeIterator it = currentNode.getNodes(  );

                    while ( it.hasNext(  ) )
                    {
                        Node node = it.nextNode(  );

                        // only select nt:file and nt:folder resources
                        if ( node.isNodeType( NODE_TYPE_FILE ) || node.isNodeType( NODE_TYPE_FOLDER ) )
                        {
                            listFiles.add( nodeToRepositoryFile( node ) );
                        }
                    }

                    return listFiles;
                }
            } );
    }

    /**
     * @param strWorkspace the workspace name
     * @param strId the node id
     * @return an IRepositoryFile
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#findById(java.lang.String, java.lang.String)
     */
    public IRepositoryFile findById( String strWorkspace, final String strId )
    {
        return (IRepositoryFile) execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    Node currentNode = getNodeById( session, strId );

                    return nodeToRepositoryFile( currentNode );
                }
            } );
    }

    /**
     * Return a node by its id
     * @param session the session to use
     * @param strId the id to search
     * @return a node
     * @throws RepositoryException
     * @throws ItemNotFoundException
     */
    private Node getNodeById( Session session, String strId )
        throws ItemNotFoundException, RepositoryException
    {
        return session.getNodeByUUID( strId );
    }

    /**
     * @param strWorkspace the workspace name
     * @param strPath the absolute path to search
     * @return an IRepositoryFile
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#findByPath(java.lang.String, java.lang.String)
     */
    public IRepositoryFile findByPath( String strWorkspace, final String strPath )
    {
        return (IRepositoryFile) execute( strWorkspace,
            new JcrCallback(  )
            {
                public Object doInJcr( Session session )
                    throws IOException, RepositoryException
                {
                    Node currentNode = getNode( session, strPath );

                    return nodeToRepositoryFile( currentNode );
                }
            } );
    }

    /**
     * @param strWorkspace the workspace name
     * @param file the file to update
     * @see fr.paris.lutece.plugins.jcr.business.IRepositoryFileDAO#store(java.lang.String, fr.paris.lutece.plugins.jcr.business.IRepositoryFile)
     */
    public void store( String strWorkspace, IRepositoryFile file )
    {
        // not implemented
    }

    /**
     * Instanciate a IRepositoryFile from node informations
     * @param node the node
     * @return an IRepositoryFile associated with this node
     */
    private IRepositoryFile nodeToRepositoryFile( Node node )
    {
        JcrRepositoryFileImpl addedFile = new JcrRepositoryFileImpl(  );
        addedFile.setAbsolutePath( getAbsolutePath( node ) );
        addedFile.setExists( exists( node ) );
        addedFile.setDirectory( isDirectory( node ) );
        addedFile.setFile( isFile( node ) );
        addedFile.setName( getName( node ) );

        addedFile.setPath( addedFile.getName(  ) );
        addedFile.setResourceId( getUUID( node ) );

        if ( addedFile.isFile(  ) )
        {
            addedFile.setMimeType( getMimeType( node ) );
            addedFile.setContent( getFileContent( node ) );
            addedFile.setLastModified( getLastModified( node ) );
            addedFile.setLength( getSize( node ) );
        }

        if ( addedFile.isDirectory(  ) )
        {
            addedFile.setLastModified( getCreated( node ) );
        }

        return addedFile;
    }

    /**
     * @param node
     * @return
     */
    private String getUUID( Node node )
    {
        String result = null;

        try
        {
            result = node.getUUID(  );
        }
        catch ( RepositoryException e )
        {
            // TODO Auto-generated catch block
            e.printStackTrace(  );
        }

        return result;
    }
}
