/*
 * Copyright (c) 2002-2007, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.jcr.business;

import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.ReferenceList;

import java.io.File;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class RepositoryFileHome
{
    public static final String CONTEXT_NAME = "jsr170";
    public static final String SPRING_JSR170_JCRLIST = "jsr170.JCRlist";
    public static final String SPRING_REPOSITORY_FILE_DAO = ".repositoryFileDAO";
    public static final String SPRING_WORKSPACE_DAO = ".workspaceDAO";
    private static final Map<String, RepositoryFileHome> _singletons = new HashMap<String, RepositoryFileHome>(  );
    private IRepositoryFileDAO _dao;
    private IWorkspaceDAO _workspaceDao;

    protected RepositoryFileHome( String context )
    {
        _dao = (IRepositoryFileDAO) SpringContextService.getPluginBean( CONTEXT_NAME,
                context + SPRING_REPOSITORY_FILE_DAO );
        _workspaceDao = (IWorkspaceDAO) SpringContextService.getPluginBean( CONTEXT_NAME, context +
                SPRING_WORKSPACE_DAO );
    }

    /**
     * Returns the instance of the singleton
     * @param context the context to use, must match a declared JCR
     * @return The instance of the singleton
     */
    public static RepositoryFileHome getInstance( String context )
    {
        RepositoryFileHome singleton = _singletons.get( context );

        if ( singleton == null )
        {
            singleton = new RepositoryFileHome( context );
            _singletons.put( context, singleton );
        }

        return singleton;
    }

    /**
     * Get the files in workspace strWorkspace
     * @param strWorkspace the workspace name
     * @return a list of IRepositoryFile
     */
    public List<IRepositoryFile> getRepositoryFileList( String strWorkspace )
    {
        return getRepositoryFileList( strWorkspace, "" );
    }

    /**
     * Get the file in workspace strWorkspace in directory path
     * @param strWorkspace the workspace name
     * @param strFileId an absolute path to the directory
     * @return the IRepositoryFile
     */
    public IRepositoryFile getRepositoryFileById( String strWorkspace, String strFileId )
    {
        return _dao.findById( strWorkspace, strFileId );
    }

    /**
     * Get the file in workspace strWorkspace in directory path
     * @param strWorkspace the workspace name
     * @param path an absolute path to the directory
     * @return the IRepositoryFile
     */
    public IRepositoryFile getRepositoryFile( String strWorkspace, String path )
    {
        return _dao.findByPath( strWorkspace, path );
    }

    /**
     * Get the files in workspace strWorkspace in directory parent
     * @param strWorkspace the workspace name
     * @param parent the parent directory absolute path
     * @return a list of IRepositoryFile
     */
    public List<IRepositoryFile> getRepositoryFileList( String strWorkspace, String parent )
    {
        return _dao.listFiles( strWorkspace, parent );
    }

    /**
     * Create a file in workspace strWorkspace in directory designed by
     * path with content file
     * @param strWorkspace the workspace name
     * @param path the parent directory absolute path
     * @param file the file content
     */
    public void addFile( String strWorkspace, String path, File file )
    {
        _dao.create( strWorkspace, path, file );
    }

    /**
     * Create a directory in workspace strWorkspace at location path
     * @param strWorkspace the workspace name
     * @param path the directory absolute path
     */
    public void createFolder( String strWorkspace, String path )
    {
        _dao.create( strWorkspace, path );
    }

    /**
     * Remove element in workspace strWorkspace at path strPath
     * @param strWorkspace the workspace name
     * @param strPath the absolute path
     */
    public void removeRepositoryFile( String strWorkspace, String strPath )
    {
        _dao.delete( strWorkspace, strPath );
    }

    /**
     * Get all available workspaces
     * @return a ReferenceList with all workspace names
     */
    public ReferenceList getAvailableWorkspaces(  )
    {
        ReferenceList result = new ReferenceList(  );

        for ( String workspaceName : _workspaceDao.getAvailableWorkspaces(  ) )
        {
            result.addItem( workspaceName, workspaceName );
        }

        return result;
    }

    /**
     * Create a new workspace
     * @param strWorkspaceName the workspace name
     */
    public void createWorkspace( String strWorkspaceName )
    {
        _workspaceDao.create( strWorkspaceName );
    }

    /**
     * Get workspace informations
     * @param strWorkspace the workspace name
     * @return the workspace
     */
    public IWorkspace getWorkspace( String strWorkspace )
    {
        return _workspaceDao.findByName( strWorkspace );
    }

    /**
     * Modify workspace acces rights
     * @param workspace the workspace
     */
    public void modify( IWorkspace workspace )
    {
        _workspaceDao.store( workspace );
    }

    /**
     * Tests if the workspace strWorkspace can be accessed with roles in userRoles
     * and access of type strAccessType
     * @param strWorkspace the workspace name
     * @param strAccessType the access type, it must be in IWorkspace.AVAILABLE_ACCESS
     * @param userRoles an array of roles
     * @return true if it is accessible, false otherwise
     */
    public boolean canAccess( String strWorkspace, String strAccessType, String[] userRoles )
    {
        IWorkspace workspace = _workspaceDao.findByName( strWorkspace );

        for ( String userRole : userRoles )
        {
            if ( Arrays.binarySearch( workspace.getRoles( strAccessType ), userRole ) >= 0 )
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Get all available access, given some roles
     * @param strWorkspace the workspace name
     * @param userRoles an array of roles
     * @return a map containing access types associated with access value
     */
    public Map<String, Boolean> getAvailableAccess( String strWorkspace, String[] userRoles )
    {
        HashMap<String, Boolean> availableAccess = new HashMap<String, Boolean>(  );
        IWorkspace workspace = _workspaceDao.findByName( strWorkspace );

        for ( String strAccessType : IWorkspace.AVAILABLE_ACCESS )
        {
            for ( String userRole : userRoles )
            {
                if ( Arrays.binarySearch( workspace.getRoles( strAccessType ), userRole ) >= 0 )
                {
                    availableAccess.put( strAccessType, Boolean.TRUE );

                    break;
                }

                availableAccess.put( strAccessType, Boolean.FALSE );
            }
        }

        return availableAccess;
    }

    /**
     * @return jcr defined in Spring context
     */
    public static ReferenceList getAvailableJcr(  )
    {
        ReferenceList mapJcrNames = new ReferenceList(  );

        for ( String jcrName : (List<String>) SpringContextService.getPluginBean( CONTEXT_NAME, SPRING_JSR170_JCRLIST ) )
        {
            try
            {
                getInstance( jcrName );
                mapJcrNames.addItem( jcrName, jcrName );
            }
            catch ( Exception e )
            {
                AppLogService.info( e );
            }
        }

        return mapJcrNames;
    }

    /**
     * @return true if workspace can be created in this jcr
     */
    public boolean canCreateWorkspace(  )
    {
        return _workspaceDao.canCreate(  );
    }

    /**
     * Removes a workspace
     * @param strWorkspaceName the name of the workspace to remove
     */
    public void removeWorkspace( String strWorkspaceName )
    {
        _workspaceDao.delete( strWorkspaceName );
    }
}
